"use strict";
this.name               = "market_observer";
this.author               = "spara";
this.copyright            = "2014 Mika Spåra";
this.description         = "Gather market info";
this.version            = "2.2.9";
this.licence     = "CC BY-NC-SA 3.0";

this.startUp = function() {
	
	//User options start here
	
	//toggleable market screen elements start here
	this.$gathererFeat = true; //show gathered reference prices
	this.$inSystemTraderFeat = true; //show main station prices in other stations
	this.$percentageFeat = true; //show percentage difference to the avg price
	this.$hyperCargoFeat = true; //show hyper cargo units	
	this.$purchaseLogFeat = true; //show purchase log
	//toggleable market screen elements end here
	
	//gatherer options start here
	this.$basisLimit = 100 //max value of basis used for avg. After crossing, current avg is weighted with this value.
	this.$timeOut = 10; //day limit before cutting of untouched min/max values takes place.
	var resetMarketData = false; //reset gathered market data, set to true, save and set to false again.
	//gatherer options end here
	
	//User options end here
	
	//Remove incompatible Commodity Markets OXP.
	if (worldScripts["commodityMarkets"]) {
		for (prop in worldScripts["commodityMarkets"]) {
			if (prop !== 'name' && prop !== 'version') delete worldScripts["commodityMarkets"][prop];
		}
		log(this.name, "Removed incompatible Commodity Markets OXP.");
	}

	//Check for plugins
	this.$tradersRating = true;
	if (!worldScripts["mo-traders_rating"])
		this.$tradersRating = false;
	this.$commodityMarkets = true;
	if (!worldScripts["mo-commodity_markets"])
		this.$commodityMarkets = false;
	
	this.$commodities = ["food","textiles","radioactives","slaves","liquor_wines","luxuries","narcotics","computers","machinery","alloys","firearms","furs","minerals","gold","platinum","gem_stones","alien_items"];
	
	//init price gatherer
	if (resetMarketData || !missionVariables.marketObserver) {
		var i, basePrice;
		this.$gathValues = [];
		this.$gathAvgBasis = 1;
		for (i = 0; i < 17; i++) {
			basePrice = system.mainStation.market[this.$commodities[i]].price;
			this.$gathValues.push([basePrice, basePrice, basePrice]);
		}
	}
	else {
		this.$gathValues = JSON.parse(missionVariables.marketObserver);
		this.$gathAvgBasis = missionVariables.marketObserverBasis;
	}
	if (resetMarketData || !missionVariables.marketObserverTimes) {
		this.$updateTimes = [];
		for (i = 0; i < 17; i++)
			this.$updateTimes.push([clock.days, clock.days]);
	}
	else {
		this.$updateTimes = JSON.parse(missionVariables.marketObserverTimes);
	}
	
	//init purchase log
	if (!missionVariables.marketObserverBuyLog) { 
		this.$buyLog = [];
		for (i = 0; i < 17; i++)
			this.$buyLog.push([]);
	}
	else 
		this.$buyLog = JSON.parse(missionVariables.marketObserverBuyLog);
	this.$marketHud = "market_hud_1.plist";//init market hud
	this.$restoreHud = player.ship.hud;//init restore hud
	this.$restoreTargetSensitivity = player.ship.reticleTargetSensitive;//for restoring target sensivity
	this.$fallBackHud = player.ship.hud;//fallback hud, if something goes wrong in restoring the hud
	this.$latestMarket = system.mainStation.market;//for noting changes in market prices
	this.$checkMinMax = false;//for oxps that change the prices when entering market screen.
	
	//remove instructions from previous mO versions
	mission.setInstructions(null, this.name);
}

this.playerWillSaveGame = function () {
	missionVariables.marketObserver = JSON.stringify(this.$gathValues);
	missionVariables.marketObserverBuyLog = JSON.stringify(this.$buyLog);
	missionVariables.marketObserverTimes = JSON.stringify(this.$updateTimes);
	missionVariables.marketObserverBasis = this.$gathAvgBasis;
}

this.playerBoughtCargo = function(commodity, units, price) {
	if (this.$commodityMarkets) {
		var reBuy = worldScripts["mo-commodity_markets"].$playerBoughtCargo(commodity, units, price);
		units = reBuy[0];
		price = reBuy[1];
	}
	//update purchace log
	if (units > 0) {
		var commIndex = this.$commodities.indexOf(commodity);
		if (this.$buyLog[commIndex].length > 0) {//handle buying the same commodity multiple times
			if (this.$buyLog[commIndex][0][1] === price) {
				this.$buyLog[commIndex][0][0] += units;
				this.$marketMessage();
				return;
			}
		}
		this.$buyLog[commIndex].unshift([units, price]);
	}
	this.$marketMessage();
}

this.playerSoldCargo = function(commodity, units, price) {
	var i, currentElement;
	var tab = "";
	var unitCount = units;
	var commIndex = this.$commodities.indexOf(commodity);
	var profit = 0;
	//update purchace log
	while(unitCount > 0 && this.$buyLog[commIndex].length > 0) { //purge buys from log
		currentElement = this.$buyLog[commIndex].shift();
		if (currentElement[0] > unitCount) { //There is more than enough purchases in one log entry
			profit += unitCount * (price - currentElement[1]);
			currentElement[0] -= unitCount;
			this.$buyLog[commIndex].unshift(currentElement);
			unitCount = 0;
		}
		else { //There is enough or less purchases in log entry
			profit += currentElement[0] * (price - currentElement[1]);
			unitCount -= currentElement[0];
		}
	}
	//show profit if profit != 0
	if (profit !== 0) { 
		this.$stopTimer();
		this.$marketTimer.nextTime = clock.absoluteSeconds + 4;
		this.$startTimer();
		var profitLine = "                                                                      Profit by trade: "+formatCredits(profit/10, true, true )+"\n\n";//70 spaces
		this.$printValues(profitLine);
		if (this.$tradersRating)
			worldScripts["mo-traders_rating"].$reportProfit(profit);
	}
	else
		this.$marketMessage();
}

//check main station prices for average count when exiting WS
this.shipExitedWitchspace = function() {
	if (system.mainStation) this.$gatherMarketInfo(true);
}

//check station prices when docking
//write off lost cargo from the profit;
this.shipDockedWithStation = function(station) {
	this.$gatherMarketInfo(false);
	if (station.isMainStation) {
		var i, j, unitsInCargo, unitsInLog, difference, done, hcUnits;
		var profitLoss = 0;
		for (i = 0; i < 17; i++) {
			//entry or entries found from purchase log
			if (this.$buyLog[i].length > 0) {
				//check units in hypercargo
				hcUnits = 0;
				if (player.ship.equipmentStatus("EQ_HYPERCARGO") === "EQUIPMENT_OK") {
					if (i !== 13 && i !== 14 && i !== 15) {
						if (i !== 16) hcUnits = worldScripts["HyperCargo"].storedArray[i];//other items than alien items
						else hcUnits = worldScripts["HyperCargo"].storedArray[13]; //alien items
					}
				}
				//total number of units carried
				unitsInCargo = manifest[this.$commodities[i]] + hcUnits;
				done = false;
				while (!done) {
					unitsInLog = 0;
					//calculate total number of units in log
					for (j = 0; j < this.$buyLog[i].length; j++)
						unitsInLog += this.$buyLog[i][j][0];
					difference = unitsInLog - unitsInCargo;
					//check if cargo is missing
					if (difference > 0) {
						//the last log entry has more than enough units to be removed, no need to remove entries
						if (this.$buyLog[i][this.$buyLog[i].length-1][0] > difference) {
							profitLoss += this.$buyLog[i][this.$buyLog[i].length-1][1] * difference;
							this.$buyLog[i][this.$buyLog[i].length-1][0] -= difference;
							done = true;
						}
						//the last log entry has enough or less than enough units to be removed, need to remove last entry
						else {
							profitLoss += this.$buyLog[i][this.$buyLog[i].length-1][1] * difference;
							this.$buyLog[i].pop();
						}
					}
					else done = true;
				}
			}
		}
		if (profitLoss > 0) {
			if (this.$tradersRating)
				worldScripts["mo-traders_rating"].$reportProfit(-1 * profitLoss);
			player.consoleMessage(formatCredits(profitLoss/10, true, true )+" worth of cargo is lost.");
		}
	}
}

//gather market data. full = true: gather min, max & avg, full = false: gather only min & max
this.$gatherMarketInfo = function(full) {
	var i, commPrice;
	var newMarket = false;
	if (full && system.mainStation) { //update average prices
		for (i = 0; i < 17; i++) { //check if market has changed since last check
			if (system.mainStation.market[this.$commodities[i]].price !== this.$latestMarket[this.$commodities[i]].price) {
				newMarket = true;
				break;
			}
		}
		if (newMarket) { //market has changed, update average prices
			var gathAvgBasis = this.$gathAvgBasis;
			for (i = 0; i < 17; i++) {
				commPrice = system.mainStation.market[this.$commodities[i]].price;
				this.$gathValues[i][1] = (gathAvgBasis * this.$gathValues[i][1] + commPrice) / (gathAvgBasis + 1);
			}
			if (gathAvgBasis < this.$basisLimit)
				this.$gathAvgBasis = gathAvgBasis + 1;
			else this.$gathAvgBasis = this.$basisLimit;
			this.$latestMarket = system.mainStation.market;
		}
	}
	var days = clock.days;
	for (i = 0; i < 17; i++) {//update min, max prices
		if (player.ship.docked)
			commPrice = player.ship.dockedStation.market[this.$commodities[i]].price;
		else
			commPrice = system.mainStation.market[this.$commodities[i]].price;
		//when docking or entering a system, check for old update min/max values and values if needed.
		if (!this.$checkMinMax && commPrice > 0) {
			//cut value for 1 % compared to the average, if the value is outdated.
			if (days - this.$updateTimes[i][0] > this.$timeOut)
				this.$gathValues[i][0] += (this.$gathValues[i][1] - this.$gathValues[i][0])/100;
			if (days - this.$updateTimes[i][1] > this.$timeOut)	
				this.$gathValues[i][2] -= (this.$gathValues[i][2] - this.$gathValues[i][1])/100;
		}
		if (commPrice >= this.$gathValues[i][2]) { //update max price
			this.$gathValues[i][2] = commPrice;
			this.$updateTimes[i][1] = days;
		}
		else if (commPrice <= this.$gathValues[i][0] && commPrice > 0) { //update min price, treat 0 as no price
			this.$gathValues[i][0] = commPrice;
			this.$updateTimes[i][0] = days;
		}
		if (this.$commodityMarkets && player.ship.docked) {
			commPrice = worldScripts["mo-commodity_markets"].$buyPrices[i];
			if (commPrice >= this.$gathValues[i][2]) { //update max price
				this.$gathValues[i][2] = commPrice;
				this.$updateTimes[i][1] = days;
			}
		}
	}
	if (this.$checkMinMax)
		this.$printValues("\n\n");
}

this.guiScreenChanged = function(to, from) {
	if (!player.ship.docked) return;
	if (guiScreen === "GUI_SCREEN_MARKET") { //swap to market hud
		this.$restoreTargetSensitivity = player.ship.reticleTargetSensitive;
		//show Commodity Markets prices or main station market prices or no extra prices
		if (this.$commodityMarkets)
			player.ship.awardEquipment("EQ_OBS_COMMODITY_MARKETS");
		else if (this.$inSystemTraderFeat && system.mainStation && !player.ship.dockedStation.isMainStation)
			player.ship.awardEquipment("EQ_OBS_MAIN_STATION");
		//show gatherer data
		if (this.$gathererFeat) player.ship.awardEquipment("EQ_OBS_GATHERER");
		//show purchase log
		if (this.$purchaseLogFeat) player.ship.awardEquipment("EQ_OBS_PURCHASES");
		//show hypercargo units
		if (this.$hyperCargoFeat && player.ship.equipmentStatus("EQ_HYPERCARGO") === "EQUIPMENT_OK") player.ship.awardEquipment("EQ_OBS_HYPERCARGO");
		//flag market prices to be checked for min & max
		this.$checkMinMax = true;
		//stop hud switching for trading assistant
		if (player.ship.equipmentStatus("EQ_AI_TRADING_ASSISTANT") !== "EQUIPMENT_OK") {
			//set ads & swap hud
			this.$restoreHud = player.ship.hud;
			var hudFileNumMinusOne = Math.floor((Math.random() * 12));
			var adNumber = hudFileNumMinusOne * 11 + Math.floor((Math.random() * 11) + 1);
			this.$randomAd = "EQ_OBS_YAH_" + adNumber;
			this.$marketHud = "market_hud_" + (hudFileNumMinusOne + 1) +".plist";
			player.ship.awardEquipment(this.$randomAd);
			//Handle hud switching throug framecallback to override other switching huds
			this.$marketFrameCallback = addFrameCallback(this.$makeSureMarketHudStaysOn.bind(this));
		}
		this.$startTimer();
	} 
	//restore hud
	if (from === "GUI_SCREEN_MARKET") {
		this.$sweepMarketHud();
		player.ship.reticleTargetSensitive = this.$restoreTargetSensitivity;
	}
}

//restore market hud, if some other oxp overrides it.
this.$makeSureMarketHudStaysOn = function(delta) {
	if (guiScreen !== "GUI_SCREEN_MARKET") {
		removeFrameCallback(this.$marketFrameCallback);
		delete this.$marketFrameCallback;
		return;
	}
	if (player.ship.hud !== this.$marketHud)
		player.ship.hud = this.$marketHud;
}

this.$sweepMarketHud = function() { //restore hud & remove timer
	if (player.ship.hud === this.$marketHud) {
		player.consoleMessage("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
		if (this.$restoreHud !== this.$marketHud)
			player.ship.hud = this.$restoreHud;
		else //if something has gone wrong and restorehud is the same as markethud. Then fall back to fallbackhud
			player.ship.hud = this.$fallBackHud;
		if (this.$randomAd) player.ship.removeEquipment(this.$randomAd);//remove ad-equipments
		player.ship.removeEquipment("EQ_OBS_COMMODITY_MARKETS");
		player.ship.removeEquipment("EQ_OBS_MAIN_STATION");
		player.ship.removeEquipment("EQ_OBS_GATHERER");
		player.ship.removeEquipment("EQ_OBS_PURCHASES");
		player.ship.removeEquipment("EQ_OBS_HYPERCARGO");
	}
	//this is for f8 mission screen timer launch behaviour
	this.$removeTimer();
}

this.$marketMessage = function() { //timer function for market screen
	if (guiScreen === "GUI_SCREEN_MARKET") {
		if (this.$checkMinMax) {//for Commodity Markets etc. Note new prices when opening the market screen.
			this.$gatherMarketInfo(false);
			this.$checkMinMax = false;
		}
		else this.$printValues("\n\n");
	}
	else this.$sweepMarketHud();
}

this.$formatCredits = function(credits) {
	if (credits < 99.5) 
		return "    " + formatCredits(credits/10, true, false);
	else if (credits < 999.5)
		return "  " + formatCredits(credits/10, true, false);
	else return formatCredits(credits/10, true, false);
}

this.$formatUnits = function(units) {
	if (units < 10) return "  " + units;
	else return units;
}

this.$percentDiff = function(commIndex) {
	var gathValue = this.$gathValues[commIndex][1];
	var percent = (player.ship.dockedStation.market[this.$commodities[commIndex]].price - gathValue) / gathValue * 100;
	if (percent < -0.05) {
		if (percent > -9.95)
			return "     " + percent.toFixed(1) + " %";
		else if (percent > -99.95)
			return "   " + percent.toFixed(1) + " %";
		else return " " + percent.toFixed(1) + " %";
	}
	else if (percent > 0.05) {
		if (percent < 9.95)
			return "    +" + percent.toFixed(1) + " %";
		else if (percent < 99.95)
			return "  +" + percent.toFixed(1) + " %";
		else return "+"+percent.toFixed(1) + " %";
	}
	else return "      0.0 %";
}

this.$printValues = function(append) { //min, avg, max, purchases formatting & printing to screen
	var i, j, pricePadding, logUnits, logValue, valuesLine, hcUnits;
	var message = "\n";
	for (i = 0; i < 17; i++) { //here we go
		valuesLine ="";
		//columns for gathered price references
		if (this.$gathererFeat) {
			for (j = 0; j < 3; j++)
				valuesLine += this.$formatCredits(this.$gathValues[i][j]) + " ";
		}
		else valuesLine = "                              ";
		//extra prices column for commodity markets
		if (this.$commodityMarkets)
			valuesLine +=  "             "  + this.$formatCredits(worldScripts["mo-commodity_markets"].$buyPrices[i])+"    ";
		//show main station prices when visiting other stations
		else if (player.ship.docked && this.$inSystemTraderFeat && system.mainStation && !player.ship.dockedStation.isMainStation)
			valuesLine += "             "  + this.$formatCredits(system.mainStation.market[this.$commodities[i]].price)+"      ";
		//show percentage difference to the avg price if not showing commodity markets or system trader feat
		else if (this.$percentageFeat) valuesLine += "           " + this.$percentDiff(i);//11 spaces
		else valuesLine += "                          ";
		//column for hyper cargo
		if (this.$hyperCargoFeat && player.ship.equipmentStatus("EQ_HYPERCARGO") === "EQUIPMENT_OK") {
			hcUnits = 0;
			if (i !== 13 && i !== 14 && i !== 15) {
				if (i !== 16) hcUnits = worldScripts["HyperCargo"].storedArray[i];//other items than alien items
				else hcUnits = worldScripts["HyperCargo"].storedArray[13]; //alien items
			}
			if (hcUnits === 0) //no hyper cargo found
				valuesLine += "                       --                  ";
			else //hyper cargo found
				valuesLine += "                      " + this.$formatUnits(hcUnits) + "                 ";
		}
		else valuesLine += "                                           ";
		//column for purchase log
		if (this.$purchaseLogFeat && this.$buyLog[i].length > 0) {
			pricePadding = "";
			logValue = this.$buyLog[i][0][1];
			if (logValue < 100) pricePadding = "    ";
			else if (logValue < 1000) pricePadding = "  "; 			
			valuesLine += pricePadding + this.$formatUnits(this.$buyLog[i][0][0]) + " x " + formatCredits(logValue/10, true, false);
			if (this.$buyLog[i].length > 1) //more that one entry in log
				valuesLine += " >";
			else valuesLine += "   ";
		}
		else //no show or no entries in purchase log
			valuesLine += "                    ";//20 spaces
		message += valuesLine + "\n";
	}
	player.consoleMessage(message,10);
	player.consoleMessage(append);
}

this.shipWillLaunchFromStation = function(station) {
	this.$sweepMarketHud();
	var i;
	for (i = 0; i < 17; i++) {//sort purchace log descending
		if (this.$buyLog[i].length > 1) 
			this.$buyLog[i].sort(function(a, b){return b[1] - a[1]});
	}
}

//Timer functions
this.$startTimer = function() {
	if (!this.$marketTimer)
		this.$marketTimer = new Timer(this, this.$marketMessage, 0, 8);
	else this.$marketTimer.start();
};

this.$stopTimer = function() {
	if (!this.$marketTimer)
		this.$marketTimer = new Timer(this, this.$marketMessage, 0, 8);
	this.$marketTimer.stop();
};

this.$removeTimer = function() {
	if (this.$marketTimer) {
		this.$marketTimer.stop();
		delete this.$marketTimer;
	}
};
